@@ -1,5 +1,6 @@
 # 2022-2-13  cp from cikuu/bin/es.py 
-import json,fire,sys, os, hashlib ,time 
+import json,fire,sys, os, hashlib ,time , requests
+from collections import Counter , defaultdict
 import warnings
 warnings.filterwarnings("ignore")
 
@@ -30,9 +31,16 @@ def index_doc(did, doc):
 	return arr
 
 from so import * 
+cursor_sql = lambda query, cursor: requests.post(f"http://{requests.eshost}:{requests.esport}/_sql", json={"query":query, "cursor":cursor}).json() #  move to so/__init__ later
+
 class ES(object):
-	def __init__(self, host='127.0.0.1',port=9200): 
-		self.es = Elasticsearch([ f"http://{host}:{port}" ])  
+	def __init__(self, eshost='127.0.0.1',esport=9200): 
+		self.host = eshost # to be removed later 
+		self.port = esport 
+		self.es	  = Elasticsearch([ f"http://{eshost}:{esport}" ])  
+		requests.eshost	= eshost
+		requests.esport	= esport
+		requests.es		= Elasticsearch([ f"http://{requests.eshost}:{requests.esport}" ])  
 
 	def addfolder(self, folder:str, pattern=".txt", idxname=None): 
 		''' folder -> docbase, 2022.1.23 '''
@@ -106,29 +114,6 @@ class ES(object):
 			self.es.index(index=idxname, body={"_snt":snt,  "sntvec": vec, 'type':'sntvec'}, id = f"{sid}-sntvec")
 		print("sntvec finished:", idxname) 
 
-	def vecso(self, idxname, snt): 
-		''' search with snt vec, 2022.3.27	'''
-		from sentence_transformers import SentenceTransformer
-		if not hasattr(fire, 'model'):  fire.model = SentenceTransformer('all-MiniLM-L6-v2')
-		vec	= fire.model.encode(snt.strip()).tolist()
-		res = self.es.search(index=idxname, body={
-  "query": {
-    "script_score": {
-      "query": {
-        "match_all": {}
-      },
-      "script": {
-        "source": "cosineSimilarity(params.query_vector, 'sntvec') + 1.0",
-        "params": {
-          "query_vector": vec
-        }
-      }
-    }
-  }
-}
-)
-		print (res) 
-
 	def propbank(self, idxname): 
 		'''  add flair semantic tag into snt, 2022.3.25
 		python -m so propbank testdoc
@@ -163,7 +148,7 @@ class ES(object):
 	def dumpid(self, idxname): [print (doc['_id'] + "\t" + json.dumps(doc['_source']))  for doc in helpers.scan(self.es,query={"query": {"match_all": {}}}, index=idxname)]
 	def dumpraw(self, idxname): [print (json.dumps(doc))  for doc in helpers.scan(self.es,query={"query": {"match_all": {}}}, index=idxname)]
 	def keys(self, idxname): [print(resp['_id']) for resp in  helpers.scan(client=self.es, query={"query" : {"match_all" : {}}}, scroll= "10m", index= idxname , timeout="10m") ]
-	def hello(self): print (self.es)
+	def hello(self): print (requests.es)
 
 	def dump(self, idxname): 
 		''' python -m so dump gzjc  > gzjc.esjson '''
@@ -172,6 +157,63 @@ class ES(object):
 			del doc["_score"]
 			print (json.dumps(doc))  
 
+	def sql(self, query="select lem, count(*) cnt from dic where type = 'tok' and pos ='VERB' and lem rlike '[a-z]+' group by lem"): 
+		''' python -m so sql "select lem, count(*) cnt from gzjc where type = 'tok' and pos ='VERB' and lem rlike '[a-z]+' group by lem"  > gzjc.esjson '''
+		cursor=''
+		while True : 
+			res = requests.post(f"http://{self.host}:{self.port}/_sql", json={"query":query, "cursor":cursor}).json()
+			print (json.dumps(res['rows']))
+			cursor = res.get('cursor','') 
+			if not cursor: break			
+
+	def lemcnt(self, idxname, pos='LEX'): 
+		''' python -m so lemcnt dic --pos VERB , 2022.6.23 '''
+		si = Counter()
+		query=f"select lex, count(*) cnt from {idxname} where type = 'tok' and lex rlike '[a-zA-Z]+' group by lex" if pos == 'LEX' else f"select lem, count(*) cnt from {idxname} where type = 'tok' and lem rlike '[a-z]+' group by lem" if pos == 'LEM' else f"select lem, count(*) cnt from {idxname} where type = 'tok' and pos ='{pos}' and lem rlike '[a-z]+' group by lem"
+		cursor=''
+		while True : 
+			res = cursor_sql(query, cursor)  # res = requests.post(f"http://{self.host}:{self.port}/_sql", json={"query":query, "cursor":cursor}).json()
+			si.update( dict(res['rows']) )
+			cursor = res.get('cursor','') 
+			if not cursor: break
+		print ( json.dumps( dict({k:v for k,v in si.most_common()}, **{"_sntnum": sntnum(idxname), "_sum": sum([v for k,v in si.items()])} ) ) ) 
+
+	def trpcnt(self, idxname, dep='dobj', gpos='VERB', dpos='NOUN', inverse:bool=False): 
+		''' python -m so trpcnt dic --dep dobj , 2022.6.23 '''
+		ssi = defaultdict(Counter)
+		query=f"select gov, lem, count(*) cnt from {idxname} where type = 'tok' and pos = '{dpos}' and dep ='{dep}' and lem rlike '[a-z]+' and gov like '%_{gpos}' group by gov, lem"
+		cursor=''
+		while True : 
+			res = cursor_sql(query, cursor) 
+			[ ssi[ lem ].update({gov.split('_')[0]:cnt}) for gov, lem, cnt in res['rows'] ] if inverse else [ ssi[gov.split('_')[0] ].update({lem:cnt}) for gov, lem, cnt in res['rows'] ]
+			cursor = res.get('cursor','') 
+			if not cursor: break
+		print ( json.dumps( dict({ k: dict(v.most_common()) for k,v in ssi.items()}, **{"_sntnum": sntnum(idxname) } ) ) ) 
+
+	def poscnt(self, idxname): 
+		''' python -m so poscnt dic , 2022.6.23 '''
+		ssi = defaultdict(Counter)
+		query=f"select lem, pos, count(*) cnt from {idxname} where type = 'tok' and lem rlike '[a-z]+' group by lem, pos"
+		cursor=''
+		while True : 
+			res = cursor_sql(query, cursor) 
+			[ ssi[ lem ].update({pos:cnt}) for lem, pos, cnt in res['rows'] if pos not in ('NNP','X')] 
+			cursor = res.get('cursor','') 
+			if not cursor: break
+		print ( json.dumps( dict({ k: dict(v.most_common()) for k,v in ssi.items()}, **{"_sntnum": sntnum(idxname) } ) ) ) 
+
+	def ssi(self, query="select lem, lex, count(*) cnt from gzjc where type = 'tok' and lem rlike '[a-z]+' group by lem, lex"): 
+		''' python -m so ssi ...  > gzjc.lexcnt , 2022.6.27 '''
+		ssi = defaultdict(Counter)
+		cursor=''
+		while True : 
+			res = cursor_sql(query, cursor) 
+			[ ssi[ lem ].update({lex.lower():cnt}) for lem, lex, cnt in res['rows'] ] 
+			cursor = res.get('cursor','') 
+			if not cursor: break
+		idxname = query.split("where")[0].strip().split("from")[-1].strip()
+		print ( json.dumps( dict({ k: dict(v.most_common()) for k,v in ssi.items()}, **{"_sntnum": sntnum(idxname) } ) ) ) 
+
 	def load(self, infile, idxname=None, batch=100000, refresh:bool=True): 
 		''' python3 -m so load gzjc.esjson '''
 		if not idxname : idxname = infile.split('.')[0]
@@ -192,215 +234,4 @@ class ES(object):
 		print(">>finished " , infile, idxname )
 
 if __name__ == '__main__':
-	fire.Fire(ES)
-
-def test(): #https://elasticsearch-py.readthedocs.io/en/master/
-	doc = {
-    'author': 'kimchy',
-    'text': 'Elasticsearch: cool. bonsai cool.',
-    'timestamp': '2020-6-24',}
-	res = es.index(index="test-index", id=2, body=doc)
-	print("result", res['result'])
-
-	res = es.get(index="test-index", id=1)
-	print(res['_source'])
-
-	es.indices.refresh(index="test-index")
-
-	res = es.search(index="test-index", body={"query": {"match_all": {}}})
-	print("Got %d Hits:" % res['hits']['total']['value'])
-	for hit in res['hits']['hits']:
-		print("%(timestamp)s %(author)s: %(text)s" % hit["_source"])
-
-
-def submit(jsonfile, idxname ='docsim', host='127.0.0.1', id='eid'):
-	''' {"eid": "1", "rid": "1419931", "uid": "19704146", "snts": ["Next Monday, a foreign friend will come to meet me.", "I want to give him some representative gifts.", "The 
-first thing I want to send is tea.", "The tea culture in Sichuan is very deep.", "On the one hand, Sichuan tea has a strong aroma and a good taste.", "On the other hand
-, tea also reflects the relaxed and elegant life culture of Sichuan people.", "The second thing I want to send is the hot pot bottom.", "It is a very symbolic symbol of
- Sichuan.", "Also reflects the food culture of Scihua"]} '''
-	print(r, infile, host, idxname, flush=True)
-	es = Elasticsearch([host]) 
-	file = open(infile, 'r')
-	while True:
-		line = file.readline()
-		if not line: break; 
-		try:
-			arr = json.loads(line.strip())
-			eid = arr[id]
-			del arr[id]
-			es.index(index=idxname, id=eid, body=arr)
-		except Exception as e:# skip those speical keys 
-			print("ex:", e)	
-	file.close()
-	print(">> finished:", infile, flush=True)
-
-def submit_batch(jsonfile, idxname ='sntsim', es_host='127.0.0.1', id='eid', batch=100000):
-	from elasticsearch import helpers
-	print(r, infile, es_host, idxname, flush=True)
-	es = Elasticsearch([es_host]) 
-	actions=[]
-	file = open(infile, 'r')
-	while True:
-		line = file.readline()
-		if not line: break; 
-		try:
-			arr = json.loads(line.strip())
-			eid = arr[id]
-			del arr[id]
-			actions.append({'_op_type':'index', '_index':idxname, '_id':eid , '_source':d})
-			if len(actions) > 0  : 
-				helpers.bulk(client=es,actions=actions, raise_on_error=False)
-				actions=[]					
-		except Exception as e:# skip those speical keys 
-			print("ex:", e)	
-	file.close()
-	if len(actions) > 0  : 
-		helpers.bulk(client=es,actions=actions, raise_on_error=False)
-	print(">> finished:", infile, flush=True)
-
-
-'''
- "mappings": {
-    "_source": {
-      "includes": [
-        "*.count",
-        "meta.*"
-      ],
-      "excludes": [
-        "meta.description",
-        "meta.other.*"
-      ]
-    }
-  }
-
-	def bulk_submit(self, actions):
-		helpers.bulk(client=self.es,actions=actions) #, raise_on_error=False
-
-PUT my_index/_doc/1
-{
-  "name": "Some binary blob",
-  "blob": "U29tZSBiaW5hcnkgYmxvYg==" 
-}
-
-
-PUT maptest/_mapping/test
-{
-    "dynamic_templates": [
-        {
-            "en": {
-                "match": "*_json",
-                "match_mapping_type": "keyword",
-                "mapping": {
-                    "type": "keyword",
-                    "store": "yes",
-                    "index": "not_analyzed"
-                }
-            }
-        }
-    ]
-}
-
-PUT /my_index
-{
-    "mappings": {
-        "my_type": {
-            "dynamic_templates": [
-                { "es": {
-                      "match":              "*_es", 
-                      "match_mapping_type": "string",
-                      "mapping": {
-                          "type":           "string",
-                          "analyzer":       "spanish"
-                      }
-                }},
-                { "en": {
-                      "match":              "*", 
-                      "match_mapping_type": "string",
-                      "mapping": {
-                          "type":           "string",
-                          "analyzer":       "english"
-                      }
-                }}
-            ]
-}}}
-https://www.elastic.co/guide/cn/elasticsearch/guide/current/custom-dynamic-mapping.html
-
-GET /eevsim/_search
-{
-  "query": {
-    "bool": {
-    "must_not":{"term" : {"uid": "123"} },  
-    "should": [
-        {
-          "match": {
-            "md5": {
-              "query": "201f2e517903766f7c41b90ef27f346f",
-              "boost": 2
-            }
-          }
-        },
-        {
-          "match": {
-            "md5": {
-              "query": "35179a54ea587953021400eb0cd23201", 
-              "boost": 3
-            }
-          }
-        }
-      ],
-       
-      "minimum_should_match": "90%"
-    }
-  }
-}
-
-{"_id": "3-20", "_source": {"rid": 11386, "uid": 21, "sc": 13, "md5": "0f4d9831fee517c2b4223e4ebf6d0930 bfd2f212e4fdbc1b4b9e3c4d14fa42b1 20b30ac261d74904e7b9b3e2cf313728 a0
-04c0a2b3e048968cea61002c71b1c6 83f565eac47ede4b022a77cd08d85bc5 a269cf9ee7c15792566a9c4953aed1a6 f380b022b086b477e6f1f6882fcf3b07 fa74e6c1cded62fad1c99999b6d613e0 c4da8d100
-6203cb7ba6103ec55c12edc 7af1c8021855d7605e6e7b3d1c370372 d33a962163197dda27c395925a2d1d48 232d21f3b22d6d6fbb6b7fd035e03053 c9dbf1351587627236c0893aac2d1573"}}
-
-12.2 , wrong program's id range 
-127.0.0.1:9221[2]> zrevrange ids 0 10
- 1) "811997267"
- 2) "811997259"
- 3) "811997255"
- 4) "811997251"
- 5) "811997231"
- 6) "811997229"
- 7) "811997215"
- 8) "811997213"
- 9) "811997195"
-10) "811997191"
-11) "811997185"
-
-if spacy: 
-	from spacy.lang import en
-	spacy.sntbr		= (inst := en.English(), inst.add_pipe("sentencizer"))[0]
-	spacy.snts		= lambda essay: [ snt.text.strip() for snt in  spacy.sntbr(essay).sents]
-	spacy.nlp		= spacy.load('en_core_web_sm')
-	spacy.frombs	= lambda bs: list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
-	spacy.tobs		= lambda doc: ( doc_bin:= spacy.tokens.DocBin(), doc_bin.add(doc), doc_bin.to_bytes())[-1]
-	spacy.vplem		= lambda doc,ibeg,iend: doc[ibeg].lemma_ + " " + doc[ibeg+1:iend].text.lower()
-	spacy.getdoc	= lambda snt: ( bs := redis.bs.get(snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), redis.bs.setnx(snt, spacy.tobs(doc)) if not bs else None )[1]
-	spacy.sntdocs	= lambda hkey: [spacy.getdoc(snt) for snt in json.loads(redis.r.hget(hkey,'snts'))] #'doc:inau:inau/1893-Cleveland.txt'
-
-2021.12.12
-ubuntu@ubuntu:/ftp/esmd5$ nohup python3 -m cikuu.bin.es loadjson eev2021.esmd5 eevsim & 
-[1] 47096
-
- "_doc": {
-				  "dynamic_templates": [
-					{
-					  "strings_as_keywords": { # only map string to keyword, not text 
-						"match_mapping_type": "string",
-						"mapping": {
-						  "type": "keyword"
-						}
-					  }
-					}
-				  ]
-				},
-			
-		print(">>finished " , idxname, file=sys.stderr)
-
-self.es.index(index=idxname,id = f"{corpus}-{file}",  body = {"filename":file,'corpus':corpus,'body':open(f"{folder}/{file}",'r', encoding='utf-8').read().strip() })
-'''
\ No newline at end of file
+	fire.Fire(ES)
\ No newline at end of file
